# -*- coding: utf8 -*-
"""
.. module:: burpui.api.custom.inputs
    :platform: Unix
    :synopsis: Burp-UI api custom inputs module.

.. moduleauthor:: Ziirish <hi+burpui@ziirish.me>


"""
from flask_restplus.inputs import *  # noqa # pylint: disable=locally-disabled, wildcard-import, unused-wildcard-import
from .my_inputs import boolean  # noqa
