
          <div id="navbar-config">
            <h4>{{ _('User settings navigation') }}</h4>
            <ul class="nav nav-sidebar bui-scrollspy">
              <li class="active" data-target="#sessions"><a href="#sessions">{{ _('Sessions') }}</a></li>
              <li data-target="#prefs"><a href="#prefs">{{ _('Preferences') }}</a></li>
              <!-- NOT YET IMPLEMENTED
              <li data-target="#acl"><a href="#acl">{{ _('ACL') }}</a></li>
              -->
            </ul>
          </div>
