    <div class="navbar navbar-inverse navbar-fixed-top" role="navigation">
      <div class="container-fluid">
        <div class="navbar-header">
          <button type="button" class="navbar-toggle" data-toggle="collapse" data-target=".navbar-collapse">
            <span class="sr-only">{{ _('Toggle navigation') }}</span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
          </button>
          <a class="navbar-brand" href="{{ url_for('view.home') }}">Burp-UI</a>
        </div>
        <div class="navbar-collapse collapse">
          <ul class="nav navbar-nav navbar-right">
            <li class="detail {% if about %}active{% endif %}">
              <a href="{{ url_for('view.about') }}">
                <span class="glyphicon glyphicon-question-sign"></span><span class="dtl">&nbsp;{{ _('About') }}</span>
              </a>
            </li>
            {% if not login -%}
              {% if not config.STANDALONE -%}
            <li class="detail {% if not server and not about and not me and not admin %}active{% endif %}">
              <a href="{{ url_for('view.home') }}">
                <span class="glyphicon glyphicon-hdd"></span><span class="dtl">&nbsp;{{ _('Servers') }}</span>
              </a>
            </li>
                {% if server -%}
            <li class="detail {% if clients %}active{% endif %}">
              <a href="{{ url_for('view.clients', server=server) }}">
                <span class="glyphicon glyphicon-tasks"></span><span class="dtl">&nbsp;Clients</span>
              </a>
            </li>
                {% endif -%}
        {% else -%}
            <li class="detail {% if clients %}active{% endif %}">
              <a href="{{ url_for('view.home') }}">
                <span class="glyphicon glyphicon-tasks"></span><span class="dtl">&nbsp;Clients</span>
              </a>
            </li>
        {% endif -%}
        {% if config.STANDALONE or server or me -%}
            <li class="detail {% if settings %}active{% endif %}">
              <a href="{{ url_for('view.settings', server=server) }}">
                <span class="glyphicon glyphicon-wrench"></span><span class="dtl">&nbsp;{{ _('Settings') }}</span>
              </a>
            </li>
              {% endif -%}
            <li class="detail {% if live %}active{% endif %}">
              <a href="{{ url_for('view.live_monitor', server=server) }}">
                <span id="toblink" class="glyphicon glyphicon-screenshot"></span><span class="dtl">&nbsp;{{ _('Live monitor') }}</span>
              </a>
            </li>
              {% if current_user and current_user.is_authenticated -%}
                {% if current_user.is_admin -%}
            <!--
            <li class="detail {% if admin %}active{% endif %}">
              <a href="{{ url_for('view.admin') }}">
                <span class="glyphicon glyphicon-cog"></span><span class="dtl">&nbsp;{{ _('Administration') }}</span>
              </a>
            </li>
            -->
                {% endif -%}
            <li class="detail {% if me %}active{% endif %}">
              <a href="{{ url_for('view.me') }}">
                <span class="glyphicon glyphicon-user"></span><span class="dtl">&nbsp;{{ _('User settings') }}</span>
              </a>
            </li>
            <li class="detail">
              <a href="{{ url_for('view.logout') }}">
                <span class="glyphicon glyphicon-log-out"></span><span class="dtl">&nbsp;{{ _('Logout') }}<small>({{ current_user.name }})</small></span>
              </a>
            </li>
              {% endif -%}
            <li {% if live %}ng-click="refresh"{% endif %}>
              <a id="refresh" href="#">
                <span class="glyphicon glyphicon-refresh"></span><span class="hidden-md hidden-lg">&nbsp;{{ _('Refresh') }}</span>
              </a>
            </li>
            {% endif -%}
          </ul>
          {% if not login -%}
          <form class="navbar-form navbar-right" id="search">
            <input type="text" class="form-control" id="input-client" placeholder="{{ _('Search client...') }}" autocomplete="off">
          </form>
          {% endif -%}
        </div>
      </div>
    </div>
