{% extends "layout.html" %}
{% block body %}
        {% include "notifications.html" %}
        <div class="col-sm-9 col-sm-offset-3 col-md-10 col-md-offset-2 main">
          {% include "small_topbar.html" %}
          <ul class="breadcrumb" style="margin-bottom: 5px;">
            {% if server -%}
            <li><a href="{{ url_for('view.home') }}">Home</a></li>
            <li><a href="{{ url_for('view.clients', server=server) }}">{{ server }} clients</a></li>
            <li><a href="{{ url_for('view.client', name=cname, server=server) }}">{{ cname }} overview</a></li>
            <li class="active">Backup n°{{ nbackup|mypad }} overview</li>
            {% else -%}
            <li><a href="{{ url_for('view.home') }}">Home</a></li>
            <li><a href="{{ url_for('view.client', name=cname) }}">{{ cname }} overview</a></li>
            <li class="active">Backup n°{{ nbackup|mypad }} overview</li>
            {% endif -%}
          </ul>
          <br />
          <h1 class="page-header">Backup n°{{ nbackup|mypad }} of {{ cname }}</h1>
          <div class="row placeholders">
            <div class="col-xs-6 col-sm-6 placeholder">
              <div class="mycharts" id="chart_new" style="height: 350px;">
                <svg></svg>
              </div>
              <h4>New</h4>
            </div>
            <div class="col-xs-6 col-sm-6 placeholder">
              <div class="mycharts" id="chart_changed" style="height: 350px;">
                <svg></svg>
              </div>
              <h4>Changed</h4>
            </div>
            <div class="col-xs-6 col-sm-6 placeholder">
              <div class="mycharts" id="chart_unchanged" style="height: 350px;">
                <svg></svg>
              </div>
              <h4>Unchanged</h4>
            </div>
            <div class="col-xs-6 col-sm-6 placeholder">
              <div class="mycharts" id="chart_deleted" style="height: 350px;">
                <svg></svg>
              </div>
              <h4>Deleted</h4>
            </div>
            <div class="col-xs-6 col-sm-6 placeholder">
              <div class="mycharts" id="chart_total" style="height: 350px;">
                <svg></svg>
              </div>
              <h4>Total</h4>
            </div>
            <div class="col-xs-6 col-sm-6 placeholder">
              <div class="mycharts" id="chart_scanned" style="height: 350px;">
                <svg></svg>
              </div>
              <h4>Scanned</h4>
            </div>
            <!--
            <div class="col-xs-12 col-sm-12 placeholder">
              <table class="table table-striped table-hover" id="table-backup">
                <thead>
                  <tr>
                    <th></th>
                    <th>New</th>
                    <th>Changed</th>
                    <th>Unchanged</th>
                    <th>Deleted</th>
                    <th>Total</th>
                    <th>Scanned</th>
                  </tr>
                </thead>
                <tbody>
                </tbody>
              </table>
            </div>
            -->
            <div class="col-xs-12 col-sm-12 placeholder">
              <div class="mycharts" id="chart_combined" style="height: 450px;">
                <svg></svg>
              </div>
            </div>
          </div>
        </div>
{% endblock %}
