{% extends "layout.html" %}
{% block body %}
        {% include "notifications.html" %}
        <div class="col-sm-9 col-sm-offset-3 col-md-10 col-md-offset-2 main">
          {% include "small_topbar.html" %}
          <ul class="breadcrumb" style="margin-bottom: 5px;">
            <li class="active">{{ _('Administration') }}</li>
          </ul>
          <br />
          <h1 class="page-header">nyan</h1>
          <div class="row placeholders">
          </div>
        </div>
{% endblock %}
