{% extends "layout.html" %}
{% block body %}
        {% include "notifications.html" %}
        <div class="col-sm-9 col-sm-offset-3 col-md-10 col-md-offset-2 main">
          {% include "small_topbar.html" %}
          <ul class="breadcrumb" style="margin-bottom: 5px;">
            {% if server -%}
            <li><a href="{{ url_for('view.home') }}">{{ _('Home') }}</a></li>
            <li class="active">{{ server }} clients</li>
            {% else -%}
            <li class="active">{{ _('Home') }}</li>
            {% endif -%}
          </ul>
          <br />
          <h1 class="page-header">Clients</h1>

          <h2 class="sub-header">Status</h2>
          <div class="table-responsive">
            <table class="table table-striped table-hover nowrap" id="table-clients" width="100%">
              <thead>
                <tr>
                  <th>{{ _('Name') }}</th>
                  <th class="desktop">{{ _('State') }}</th>
                  <th class="desktop">{{ _('Last Backup') }}</th>
                </tr>
              </thead>
              <tbody>
              </tbody>
            </table>
          </div>
        </div>
{% endblock %}
