# -*- coding: utf8 -*-
"""
.. module:: burpui.ext.session
    :platform: Unix
    :synopsis: Burp-UI external session module.

.. moduleauthor:: Ziirish <hi+burpui@ziirish.me>

"""
from flask_session import Session

sess = Session()
