{% extends "layout.html" %}
{% import 'macros.html' as macros %}
{% block body %}
        {% include "notifications.html" %}
        <div class="main">
          <div class="wrapper">
            {% call macros.render_form(form, action_text=_('Login'), class_='form-signin', btn_class='btn btn-info btn-lg btn-primary btn-block') %}
              {{ macros.render_field(form.username, placeholder=_('Username'), type='text', autofocus=true) }}
              {{ macros.render_field(form.password, placeholder=_('Password'), type='password') }}
              {{ macros.render_field(form.language) }}
              {{ macros.render_checkbox_field(form.remember) }}
            {% endcall %}
          </div>
        </div>
{% endblock %}
