
          <div id="navbar-config">
            <h4>{{ _('Administration navigation') }}</h4>
            <ul class="nav nav-sidebar bui-scrollspy">
              <li class="active" data-target="#users"><a href="#users">{{ _('Users') }}</a></li>
              <li data-target="#settings"><a href="#settings">{{ _('Settings') }}</a></li>
              <li data-target="#acl"><a href="#acl">{{ _('ACL') }}</a></li>
            </ul>
          </div>
