        {% if not cname -%}
        {% set cname = None %}
        {% endif -%}
        {% if not server -%}
        {% set server = None %}
        {% endif -%}
        <div class="col-sm-3 col-md-2 sidebar">
          {% if settings -%}
          {% include "sideconfig.html" %}
          {% elif me -%}
          {% include "sideuser.html" %}
          {% elif admin -%}
          {% include "sideadmin.html" %}
          {% else -%}
          <ul class="nav nav-sidebar">
            <li {% if overview %}class="active"{% endif %}>
            {% if backup %}
            <a href="{{ url_for('view.client_browse', name=cname, server=server, backup=nbackup) }}"><span class="glyphicon glyphicon-th"></span>&nbsp;{{ _('Overview') }}</a>
            {% elif client %}
            <a href="{{ url_for('view.client', name=cname, server=server) }}"><span class="glyphicon glyphicon-th"></span>&nbsp;{{ _('Overview') }}</a>
            {% elif clients %}
            <a href="{{ url_for('view.clients', server=server) }}"><span class="glyphicon glyphicon-th"></span>&nbsp;{{ _('Overview') }}</a>
            {% elif servers %}
            <a href="{{ url_for('view.servers') }}"><span class="glyphicon glyphicon-th"></span>&nbsp;{{ _('Overview') }}</a>
            {% else %}
            <a href="#"><span class="glyphicon glyphicon-th"></span>&nbsp;{{ _('Overview') }}</a>
            {% endif %}
            </li>
            <li {% if report %}class="active"{% endif %}>
            {% if client and not backup %}
            <a href="{{ url_for('view.client_report', name=cname, server=server) }}"><span class="glyphicon glyphicon-stats"></span>&nbsp;{{ _('Reports') }}</a>
            {% elif clients %}
            <a href="{{ url_for('view.clients_report', server=server) }}"><span class="glyphicon glyphicon-stats"></span>&nbsp;{{ _('Reports') }}</a>
            {% elif backup %}
            <a href="{{ url_for('view.backup_report', name=cname, backup=nbackup, server=server) }}"><span class="glyphicon glyphicon-stats"></span>&nbsp;{{ _('Reports') }}</a>
            {% elif servers %}
            <a href="{{ url_for('view.servers_report') }}"><span class="glyphicon glyphicon-stats"></span>&nbsp;{{ _('Reports') }}</a>
            {% else %}
            <a href="#"><span class="glyphicon glyphicon-stats"></span>&nbsp;{{ _('Reports') }}</a>
            {% endif %}
            </li>
            <li {% if calendar %} class="active"{% endif %}>
              <a href="{{ url_for('view.calendar', client=cname, server=server) }}"><span class="glyphicon glyphicon-calendar"></span>&nbsp;{{ _('Calendar') }}</a>
            </li>
          </ul>
          {% endif -%}
        </div>
